@@ -3,24 +3,22 @@ from __future__ import with_statement
 import copy
 from functools import partial
 from operator import isMappingType
-import os
+import os.path
 import sys
-from contextlib import contextmanager
 
-from fudge import Fake, patched_context, with_fakes
+from fudge import Fake, patched_context
 from nose.tools import ok_, eq_
 
 from fabric.decorators import hosts, roles, task
 from fabric.context_managers import settings
-from fabric.main import (parse_arguments, _escape_split,
+from fabric.main import (parse_arguments, _escape_split, find_fabfile,
         load_fabfile as _load_fabfile, list_commands, _task_names,
         COMMANDS_HEADER, NESTED_REMINDER)
 import fabric.state
-from fabric.state import _AttributeDict
 from fabric.tasks import Task, WrappedCallableTask
 from fabric.task_utils import _crawl, crawl, merge
 
-from utils import mock_streams, eq_, FabricTest, fabfile, path_prefix, aborts
+from utils import FabricTest, fabfile, path_prefix, aborts
 
 
 # Stupid load_fabfile wrapper to hide newly added return value.
@@ -362,6 +360,33 @@ def test_lazy_roles():
     eq_hosts(command, ['a', 'b'], env={'roledefs': lazy_role})
 
 
+#
+# Fabfile finding
+#
+
+class TestFindFabfile(FabricTest):
+    """Test Fabric's fabfile discovery mechanism."""
+    def test_find_fabfile_can_discovery_package(self):
+        """Fabric should be capable of loading a normal package."""
+        path = self.mkfile("__init__.py", "")
+        name = os.path.dirname(path)
+        assert find_fabfile([name,]) is not None
+
+    def test_find_fabfile_can_discovery_package_with_pyc_only(self):
+        """
+        Fabric should be capable of loading a package with __init__.pyc only.
+        """
+        path = self.mkfile("__init__.pyc", "")
+        name = os.path.dirname(path)
+        assert find_fabfile([name,]) is not None
+
+    def test_find_fabfile_should_refuse_fake_package(self):
+        """Fabric should refuse to load a non-package directory."""
+        path = self.mkfile("foo.py", "")
+        name = os.path.dirname(path)
+        assert find_fabfile([name,]) is None
+
+
 #
 # Fabfile loading
 #
@@ -497,7 +522,6 @@ class TestNamespaces(FabricTest):
         Wrapped new-style tasks should preserve their function names
         """
         module = fabfile('decorated_fabfile_with_classbased_task.py')
-        from fabric.state import env
         with path_prefix(module):
             docs, funcs = load_fabfile(module)
             eq_(len(funcs), 1)
@@ -509,7 +533,6 @@ class TestNamespaces(FabricTest):
         variable name.
         """
         module = fabfile('classbased_task_fabfile.py')
-        from fabric.state import env
         with path_prefix(module):
             docs, funcs = load_fabfile(module)
             eq_(len(funcs), 1)
